/*! @file mainpage.h

This file contains all documentation, no code, in the form of comment text.  It provides
chapter 1 of the documentation that is produced with doxygen. Chapter 1 includes the title page, installation instructions, and prose on the nature of FFT and their use in our library.

@mainpage FluidFFT

The FluidFFT package provides C++ classes and their Python wrapper classes useful to perform discrete Fast Fourier Transform (FFT) with different libraries, in particular 
@li fftw3 and fftw3-mpi
@li pfft
@li p3dfft
@li cufft (fft library by CUDA running on GPU)

This page contains only the documentation of the C++ classes.

@section pfft pfft class
@section p3dfft p3dfft class
@section cufft cufft class


*/
