
class FFT2dFakeForDoc(object):
    """Class to perform Fast Fourier Transform in 2d."""
    def __init__(self, n0=2, n1=2):
        pass

    def get_local_size_X(self):
        pass

    def run_tests(self):
        """Run the c++ tests."""

    def run_benchs(self, nb_time_execute=10):
        """Run the c++ benchmarcks"""

    def fft_as_arg(self, fieldX, fieldK):
        """Perform the fft and copy the result in the second argument."""

    def ifft_as_arg(self, fieldK, fieldX):
        """Perform the ifft and copy the result in the second argument."""

    def fft(self, fieldX):
        """Perform the fft and returns the result."""

    def ifft(self, fieldK):
        """Perform the ifft and returns the result."""

    def get_shapeX_loc(self):
        pass

    def get_shapeK_loc(self):
        pass

    def get_shapeX_seq(self):
        pass

    def get_shapeK_seq(self):
        pass

    def get_is_transposed(self):
        """Get the boolean "is_transposed"."""

    def get_seq_indices_first_X(self):
        pass

    def get_seq_indices_first_K(self):
        pass

    def get_k_adim_loc(self):
        pass

    def get_x_adim_loc(self):
        pass

    def compute_energy_from_X(self, fieldX):
        pass

    def compute_energy_from_K(self, fieldK):
        pass
